@@ -18,11 +18,11 @@ ZConf::Mail - Misc mail client functions backed by ZConf.
 
 =head1 VERSION
 
-Version 0.0.0
+Version 0.1.0
 
 =cut
 
-our $VERSION = '0.0.0';
+our $VERSION = '0.1.0';
 
 
 =head1 SYNOPSIS
@@ -90,32 +90,34 @@ sub new {
 
 	#this defines legal values for later use
 	$self->{legal}{pop3}=['user', 'pass', 'auth', 'useSSL',
-						  'deliverTo', 'fetchable', 'server', 'port'];
-	$self->{legal}{imap}=['user', 'pass', 'useSSL','deliverTo',
-						  'fetchable', 'inbox', 'server', 'port'];
-	$self->{legal}{mbox}=['mbox', 'deliverTo', 'fetchable'];
+						  'deliverTo', 'deliverToFolder', 'fetchable',
+						  'server', 'port'];
+	$self->{legal}{imap}=['user', 'pass', 'useSSL','deliverTo', 'deliverToFolder',
+						  'fetchable', 'inbox', 'server', 'port', 'timeout'];
+	$self->{legal}{mbox}=['mbox', 'deliverTo', 'deliverToFolder', 'fetchable'];
 	$self->{legal}{smtp}=['user', 'pass', 'auth', 'useSSL',
 						  'server', 'port', 'name', 'from',
-						  'name', 'timeout'];
-	$self->{legal}{maildir}=['maildir','deliverTo', 'fetchable'];
+						  'name', 'timeout', 'saveTo', 'saveToFolder'];
+	$self->{legal}{maildir}=['maildir','deliverTo', 'deliverToFolder', 'fetchable'];
+	$self->{legal}{exec}=['deliver'];
 
 	#this defines the required values for later use
-	#currently the same as for 
 	$self->{required}{pop3}=['user', 'pass', 'auth', 'useSSL',
 						  'deliverTo', 'fetchable', 'server', 'port'];
-	$self->{required}{imap}=['user', 'pass', 'useSSL', 'deliverTo',
+	$self->{required}{imap}=['user', 'pass', 'useSSL',
 						  'fetchable', 'inbox', 'server', 'port'];
 	$self->{required}{mbox}=['mbox', 'deliverTo', 'fetchable'];
 	$self->{required}{smtp}=['user', 'pass', 'auth', 'useSSL',
 							 'server', 'port', 'name', 'from',
-							 'name', 'timeout'];
+							 'name', 'timeout', 'saveTo'];
 	$self->{required}{maildir}=['maildir','deliverTo', 'fetchable'];
+	$self->{required}{exec}=['deliver'];
 
 	#This contains a list of account types that are fetchable.
 	$self->{fetchable}=['pop3', 'imap', 'maildir', 'mbox'];
 
 	#This contains a list of account types that are deliverable.
-	$self->{deliverable}=['exec'];
+	$self->{deliverable}=['exec','imap'];
 
 	#This contains a list of account types that are sendable.
 	#Listed for possible future purposes.
@@ -196,7 +198,7 @@ sub connectIMAP{
 	#checks to make sure all the required variables are present
 	my $requiredInt=0;
 	while (defined($self->{required}{$split[0]}[$requiredInt])) {
-		if (!defined($avars{$self->{required}{smtp}[$requiredInt]})) {
+		if (!defined($avars{$self->{required}{imap}[$requiredInt]})) {
 			warn('ZConf-Mail connectIMAP:32: "'.$self->{required}{$split[0]}[$requiredInt].
 				 '" is undefined for the account "'.$account.'"');
 			$self->{error}=32;
@@ -272,7 +274,7 @@ sub connectMaildir{
 	#checks to make sure all the required variables are present
 	my $requiredInt=0;
 	while (defined($self->{required}{$split[0]}[$requiredInt])) {
-		if (!defined($avars{$self->{required}{smtp}[$requiredInt]})) {
+		if (!defined($avars{$self->{required}{maildir}[$requiredInt]})) {
 			warn('ZConf-Mail connectMaildir:32: "'.$self->{required}{$split[0]}[$requiredInt].
 				 '" is undefined for the account "'.$account.'"');
 			$self->{error}=32;
@@ -330,7 +332,7 @@ sub connectMbox{
 	#checks to make sure all the required variables are present
 	my $requiredInt=0;
 	while (defined($self->{required}{$split[0]}[$requiredInt])) {
-		if (!defined($avars{$self->{required}{smtp}[$requiredInt]})) {
+		if (!defined($avars{$self->{required}{mbox}[$requiredInt]})) {
 			warn('ZConf-Mail connectMbox:32: "'.$self->{required}{$split[0]}[$requiredInt].
 				 '" is undefined for the account "'.$account.'"');
 			$self->{error}=32;
@@ -631,7 +633,16 @@ sub createAccount{
 		$self->{zconf}->setVar('mail', $variable, $args{$self->{required}{$args{type}}[$requiredInt]});
 
 		$requiredInt++;
-	}	
+	}
+
+	#saves it
+	$self->{zconf}->writeSetFromLoadedConfig({config=>'mail'});
+	if ($self->{zconf}->{error}) {
+		warn('ZConf-Mail createAccount:36: ZConf failed to write the set out');
+		$self->{error}=36;
+		$self->{errorString}='ZConf failed to write the set out.';
+		return undef;
+	}
 
 	return 1;
 }
@@ -736,7 +747,7 @@ sub delAccount{
 	#This implements a simple check to make sure no one passed
 	#'pop3/' or something like that. It also makes sure that
 	#nothing past the account name is refered to.
-	my @split=split(/\//, $$);
+	my @split=split(/\//, $account);
 	if (!defined($split[1]) || defined($split[3])) {
 		warn('ZConf-Mail delAccount:4: "'.$account.'" is not a valid acocunt name');
 		$self->{error}=4;
@@ -752,7 +763,16 @@ sub delAccount{
 
 	#removes the variables
 	#Off hand I can't think of a reason it should fail at this point.
-	my @deleted=$self->{zconf}->regexVarSearch('mail', '^accounts/'.$account.'/');
+	my @deleted=$self->{zconf}->regexVarDel('mail', '^accounts/'.$account.'/');
+
+	#saves it
+	$self->{zconf}->writeSetFromLoadedConfig({config=>'mail'});
+	if ($self->{zconf}->{error}) {
+		warn('ZConf-Mail delAccount:36: ZConf failed to write the set out');
+		$self->{error}=36;
+		$self->{errorString}='ZConf failed to write the set out.';
+		return undef;
+	}
 
 	return 1;
 }
@@ -810,18 +830,40 @@ sub deliverable{
 This is a wrapper function to the other deliver functions. This is a
 wrapper to the other delivery functions.
 
+The first arguement is the account. The second is the message. The
+third is the a hash that some deliver types optionally use.
+
+=head3 args hash
+
+=head4 folder
+
+This can be used to specify a folder to deliver to. If it is not
+defined, it will try to use what ever is the inbox for that account.
+
+Currently this is only used by IMAP.
+
     #delivers the mail contained in $mail to 'exec/foo'
     $zcmail->deliver('exec/foo', $mail);
     if($zcmail->{error}){
         print "deliver error";
     }
 
+    #delivers the mail contained in $mail to 'imap/foo' to the 'foo.bar'
+    $zcmail->deliver('imap/foo', $mail, {folder=>'foo.bar'});
+    if($zcmail->{error}){
+        print "deliver error";
+    }
+
 =cut
 
 sub deliver{
 	my $self=$_[0];
 	my $account=$_[1];
 	my $mail=$_[2];
+	my %args;
+	if(defined($_[3])){
+		%args= %{$_[3]};
+	}
 
 	$self->errorBlank;
 
@@ -841,6 +883,14 @@ sub deliver{
 		}		
 	}
 
+	#delivers it to a imap account
+	if ($account =~ /^imap\//) {
+		if (!$self->deliverIMAP($account, $mail, \%args)) {
+			warn('ZConf-Mail deliver:'.$self->{error}.': '.$self->{errorString});
+			return undef;
+		}
+	}
+
 	return 1;
 }
 
@@ -851,6 +901,9 @@ This is a delivers to a exec account.
 It is generally not best to call it directly, but to let the deliver
 function route it. This allows for more flexible delivery.
 
+The first arguement is the account. The second is the message. The
+third is the a optional args hash.
+
     #delivers the mail contained in $mail to 'exec/foo'
     $zcmail->deliverExec('exec/foo', $mail);
     if($zcmail->{error}){
@@ -912,6 +965,90 @@ sub deliverExec{
 	return 1;
 }
 
+=head2 deliverIMAP
+
+This is a delivers to a IMAP account.
+
+It is generally not best to call it directly, but to let the deliver
+function route it. This allows for more flexible delivery.
+
+    #delivers the mail contained in $mail to 'imap/foo' to the inbox
+    $zcmail->deliverIMAP('imap/foo', $mail);
+    if($zcmail->{error}){
+        print "deliver error";
+    }
+
+    #delivers the mail contained in $mail to 'imap/foo' to the 'foo.bar'
+    $zcmail->deliverIMAP('imap/foo', $mail, {folder=>'foo.bar'});
+    if($zcmail->{error}){
+        print "deliver error";
+    }
+
+=cut
+
+sub deliverIMAP{
+	my $self=$_[0];
+	my $account=$_[1];
+	my $mail=$_[2];
+	my %args;
+	if(defined($_[3])){
+		%args= %{$_[3]};
+	}
+
+	$self->errorBlank;
+
+	#connects to the IMAP server
+	my $imap=$self->connectIMAP($account);
+	if ($self->{error}) {
+		warn('ZConf-Mail fetchIMAP:22: Failed to connect to the IMAP server.');
+		return undef;
+	}
+
+	#delivers it to a exec account
+	if (!$account =~ /^IMAP\//) {
+		warn('ZConf-Mail deliverIMAP:19: "'.$account.'" is not a IMAP account');
+		$self->{error}=19;
+		$self->{errorString}='"'.$account.'" is not a IMAP account';
+		return undef;
+	}
+
+	#
+	if (!defined($args{folder})) {
+		#Make sure the command is defined.
+		if (!defined($self->{zconf}->{conf}{mail}{'accounts/'.$account.'/inbox'})) {
+			warn('ZConf-Mail deliverIMAP:33: "'.$account.'" is missing the variable "inbox"');
+			$self->{error}=33;
+			$self->{errorString}='"'.$account.'" is missing the variable "inbox"';
+			return undef;		
+		}
+		
+		$args{folder}=$self->{zconf}->{conf}{mail}{'accounts/'.$account.'/inbox'};
+	}
+
+	#make sure the folder exists
+	my $select=$imap->select($args{folder});
+	if (!$select) {
+		warn('ZConf-Mail deliverIMAP:34: Failed to select folder "'.$args{folder}.
+			 '" for account "'.$account.'"');
+		$self->{error}=34;
+		$self->{errorString}='Failed to select folder "'.$args{folder}.
+		                     '"for account "'.$account.'"';
+		return undef;
+	}
+
+	my $append=$imap->append($args{folder}, ['Literal', $mail]);
+	if (!$append) {
+		warn('ZConf-Mail deliverIMAP:34: Failed to append to folder "'.$args{folder}.
+			 '"for account "'.$account.'"');
+		$self->{error}=35;
+		$self->{errorString}='Failed to append to folder "'.$args{folder}.
+		                     '"for account "'.$account.'"';
+		return undef;
+	}
+
+	return 1;
+}
+
 =head2 fetch
 
 This is a wrapper function for the other accounts. The only accepted arg
@@ -1062,10 +1199,10 @@ sub fetchIMAP{
 		return undef;
 	}
 
+	#connects to the IMAP server
 	my $imap=$self->connectIMAP($account);
-
 	if ($self->{error}) {
-		warn('ZConf-Mail fetchIMAP:22: Failed to connect to the maildir.');
+		warn('ZConf-Mail fetchIMAP:22: Failed to connect to the IMAP server.');
 		return undef;
 	}
 
@@ -1329,7 +1466,7 @@ sub fetchPOP3{
 
 =head2 getAccounts
 
-Gets a array of the various SMTP accounts.
+Gets a array of the various accounts.
 
     my @accounts=$zcmail->getAccounts;
     if($zcmail->{error}){
@@ -1349,14 +1486,10 @@ sub getAccounts{
 	my %accountsH;
 
 	#goes through the list and splits it apart...
-	#each match is then the at every '/'...
-	#the split is then dumped into %accountsH as a key of the 1 and 2 positions...
-	#so for 'accounts/pop3/monkey/...', it would be 'pop3/monkey'...
-	#
 	#A hash is used for this as this operates on every key under a account... this means
 	#it will be finding multiple hits for each account
 	while (defined($matched[$matchedInt])) {
-		my @split=split(/\//, $matched[$matchedInt],3);
+		my @split=split(/\//, $matched[$matchedInt],4);
 		$accountsH{$split[1].'/'.$split[2]}='';
 		
 		$matchedInt++;
@@ -1462,6 +1595,96 @@ sub init{
 	return 1;
 }
 
+=head2 modAccount
+
+Modifies a account.
+
+=head3 args hash
+
+Outside of account type, the rest are variables that will be changed.
+
+=head4 type
+
+This is the type of a account it is. It is always lower case as can be
+seen in the variables section.
+
+=head4 account
+
+This is the name of the account. It will be appended after the account
+type. Thus for the account 'pop3/some account name' it will be
+'some account name'.
+
+=cut
+
+sub modAccount{
+	my $self=$_[0];
+	my %args;
+	if(defined($_[1])){
+		%args= %{$_[1]};
+	}
+
+	#blanks any previous errors
+	$self->errorBlank;
+
+	#make sure the type is defined
+	if (!defined($args{type})) {
+		warn('ZConf-Mail createAccount:2: $args{type} is undefined');
+		$self->{error}='2';
+		$self->{errorString}='$args{type} is undefined';
+		return undef;
+	}
+
+	#make sure it is a known type
+	if (!defined($self->{legal}{$args{type}})) {
+		warn('ZConf-Mail modAccount:3: $args{type}, "'.$args{type}.'", not a valid type');
+		$self->{error}='3';
+		$self->{errorString}='$args{type}, "'.$args{type}.'", not a valid value';
+		return undef;
+	}
+
+	#make sure it is a legit account name
+	if (!$self->{zconf}->setNameLegit($args{account})){
+		warn('ZConf-Mail modAccount:4: $args{account}, "'.$args{account}.'", not a legit account name');
+		$self->{error}='3';
+		$self->{errorString}='$args{account}, "'.$args{account}.'", not a legit account name';
+		return undef;
+	}
+
+	#puts together the accountname
+	my $account=$args{type}.'/'.$args{account};
+
+	#makes sure the account exists
+	if (!$self->accountExists($account)) {
+		warn('ZConf-Mail modAccount:6: The account "'.$account.'" does not exist');
+		$self->{error}=6;
+		$self->{errorString}='The account "'.$account.'" does not exist';
+		return undef;
+	}
+
+	#adds them
+	my $int=0;
+	while (defined($self->{legal}{$args{type}}[$int])) {
+		my $variable='accounts/'.$args{type}.'/'.$args{account}.'/'.$self->{legal}{$args{type}}[$int];
+
+		#sets the value
+		#not doing any error checking as there is no reason to believe it will fail
+		$self->{zconf}->setVar('mail', $variable, $args{$self->{legal}{$args{type}}[$int]});
+
+		$int++;
+	}
+
+	#saves it
+	$self->{zconf}->writeSetFromLoadedConfig({config=>'mail'});
+	if ($self->{zconf}->{error}) {
+		warn('ZConf-Mail modAccount:36: ZConf failed to write the set out');
+		$self->{error}=36;
+		$self->{errorString}='ZConf failed to write the set out.';
+		return undef;
+	}
+
+	return 1;
+}
+
 =head2 send
 
 This sends a email. One arguement is accepted and that is a hash.
@@ -1488,6 +1711,10 @@ This is a array of bcc addresses.
 
 This is the raw mail message to send.
 
+=head4 save
+
+This will override if a sent message will be saved or not.
+
 =cut
 
 sub send{
@@ -1569,10 +1796,12 @@ sub send{
 	}
 
 	#ends the data session
+	#Not currently doing error checking as this appears to error occasionally
+	#with out any detectable reason.
 	if (!$smtp->dataend) {
-		warn('ZConf-Mail send:29: Failed to send the data.');
-		$self->{error}=29;
-		$self->{errorString}='Failed to end the data session.';
+#		warn('ZConf-Mail send:29: Failed to end the data.');
+#		$self->{error}=29;
+#		$self->{errorString}='Failed to end the data session.';
 	}
 
 	#quits
@@ -1582,6 +1811,50 @@ sub send{
 		$self->{errorString}='Failed to end the SMTP session.';
 	}
 
+	#we know it has args if we get this far so we don't have to check it
+	my %acctArgs=$self->getAccountArgs($args{account});
+
+	#saves it if asked
+	if ($args{save}) {
+		if (!defined($acctArgs{saveTo})) {
+			warn('ZConf-Mail send:37: saveTo is undefined and thus the mail can\'t be saved');
+			$self->{error}=37;
+			$self->{errorString}='saveTo is undefined and thus the mail can\'t be saved.';
+			return undef;
+		}
+
+		if ($acctArgs{saveTo}=~ /^$/) {
+			warn('ZConf-Mail send:37: saveTo is set to "" and thus can\'t be saved');
+			$self->{error}=37;
+			$self->{errorString}='saveTo is is set to "" and thus the mail can\'t be saved';
+			return undef;
+		}
+		
+		$self->deliver($acctArgs{saveTo}, $args{mail},{folder=>$acctArgs{folder}});
+		if ($self->{error}) {
+			warn('ZConf-BGSet send: deliver errored');
+			return undef;
+		}
+
+		return 1;
+	}
+
+	#if this is not defined, we don't save it
+	if (!defined($acctArgs{saveTo})) {
+		return 1;
+	}
+
+	#if this is set, we don't save it
+	if ($acctArgs{saveTo}=~ /^$/) {
+		return 1;
+	}
+
+	$self->deliver($acctArgs{saveTo}, $args{mail},{folder=>$acctArgs{saveToFolder}});
+	if ($self->{error}) {
+		warn('ZConf-BGSet send: deliver errored');
+		return undef;
+	}
+
 	return 1;
 }
 
@@ -1808,6 +2081,15 @@ The from address to use for a account.
 
 The name that will be used with the account.
 
+=head3 accounts/smtp/*/saveTo
+
+This is the account to save it to. If it not defined
+or blank, it will not saved.
+
+=head3 accounts/smtp/*/saveToFolder
+
+This is the folder to save it to for the account.
+
 =head3 accounts/smtp/*/timeout
 
 The time out for connecting to the server.
@@ -1986,6 +2268,27 @@ Failed to create a Email::Simple object.
 
 The account has one or more missing variables.
 
+=head2 33
+
+No INBOX specified for the IMAP account.
+
+=head2 34
+
+Failed to selected IMAP folder.
+
+=head2 35
+
+Failed to append to the IMAP folder.
+
+=head2 36
+
+ZConf error.
+
+=head2 37
+
+'saveTo' is not enabled for this account. This means it is either
+undefined or set to ''.
+
 =head1 AUTHOR
 
 Zane C. Bowers, C<< <vvelox at vvelox.net> >>
